#!/usr/bin/perl -w
use strict;
use warnings;
use diagnostics;

use Test::More tests => 78;
use Test::Output;
use Test::Exception;
use Test::File;
use Path::Tiny;
use File::Copy::Recursive;  # for dircopy()
use File::Path qw(rmtree);


# *** Read before running tests ***
# Usage: perl -I lib/ t/slubsipbuilderbagit.t 
# Warning: While running project "SLUB_SIP_Builder" two directories "export_dir_kitodo/bagit" and "tmp/bagit" will be created.
# Start scipt in the project directory to run all tests properly. 

### prepare
BEGIN {
    use Path::Tiny;
    push @INC, Path::Tiny::path(__FILE__)->parent->parent->path("bin")->absolute->stringify;
    require "slubsipbuilderbagit.pl";
    $INC{'SLUB/LZA/SIPBuilderBagIt.pm'} = 1; # needed because inlined module
}
my $unpatched_mods=<<'UNPATCHED_MODS';
<?xml version="1.0" encoding="UTF-8"?>
<mods xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns="http://www.loc.gov/mods/v3" version="3.6" xsi:schemaLocation="http://www.loc.gov/mods/v3 http://www.loc.gov/standards/mods/v3/mods-3-6.xsd"><titleInfo><nonSort xml:space="preserve">Der  </nonSort><title>Fichtelberg</title><subTitle>Berg der unbekannten Rekorrde</subTitle></titleInfo><name type="personal"><namePart>Schneider, Dirk</namePart><role><roleTerm type="text">FilmemacherIn</roleTerm></role><role><roleTerm authority="marcrelator" type="code">fmk</roleTerm></role><nameIdentifier>(DE-627)1235502279 (DE-576)165502274</nameIdentifier></name><typeOfResource>moving image</typeOfResource><genre authority="rdacontent">zweidimensionales bewegtes Bild</genre><genre authority="gnd-content">Film</genre><originInfo><place><placeTerm type="code" authority="marccountry">xx</placeTerm></place><dateIssued encoding="marc">2014</dateIssued><issuance>monographic</issuance></originInfo><originInfo eventType="publication"><place><placeTerm type="text">[Leipzig]</placeTerm></place><publisher>top ten tv</publisher><dateIssued>[2014]</dateIssued></originInfo><language><languageTerm authority="iso639-2b" type="code">ger</languageTerm></language><physicalDescription><form authority="marccategory">electronic resource</form><form authority="marcsmd">remote</form><extent>1 Online-Ressource (1 Videodatei, 29:49) farbig</extent><form type="media" authority="rdamedia">Computermedien</form><form type="carrier" authority="rdacarrier">Online-Ressource</form></physicalDescription><targetAudience authority="marctarget">juvenile</targetAudience><note type="statement of responsibility" altRepGroup="00">ein Film von Dirk Schneider</note><note>Dokumentarfilm. Deutschland. 2014</note><relatedItem type="series"><titleInfo><title>MDR</title></titleInfo></relatedItem><relatedItem type="series"><titleInfo><title>Der Osten - entdecke wo du lebst</title></titleInfo></relatedItem><identifier type="oclc">946544758</identifier><recordInfo><descriptionStandard>rda</descriptionStandard><recordContentSource authority="marcorg">DE-576</recordContentSource><recordCreationDate encoding="marc">160304</recordCreationDate><recordChangeDate encoding="iso8601">20160510144338.0</recordChangeDate><recordIdentifier source="DE-576">457035137</recordIdentifier><recordOrigin>Converted from MARCXML to MODS version 3.6 using MARC21slim2MODS3-6.xsl
                (Revision 1.119 2018/06/21)</recordOrigin><languageOfCataloging><languageTerm authority="iso639-2b" type="code">ger</languageTerm></languageOfCataloging></recordInfo></mods>
UNPATCHED_MODS
my $patched_mods=<<'PATCHED_MODS';
<?xml version="1.0" encoding="UTF-8"?>
<mods xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns="http://www.loc.gov/mods/v3" version="3.6" xsi:schemaLocation="http://www.loc.gov/mods/v3 http://www.loc.gov/standards/mods/v3/mods-3-6.xsd"><titleInfo><nonSort xml:space="preserve">Der  </nonSort><title>Fichtelberg</title><subTitle>Berg der unbekannten Rekorrde</subTitle></titleInfo><name type="personal"><namePart>Schneider, Dirk</namePart><role><roleTerm type="text">FilmemacherIn</roleTerm></role><role><roleTerm authority="marcrelator" type="code">fmk</roleTerm></role><nameIdentifier>(DE-627)1235502279 (DE-576)165502274</nameIdentifier></name><typeOfResource>moving image</typeOfResource><genre authority="rdacontent">zweidimensionales bewegtes Bild</genre><genre authority="gnd-content">Film</genre><originInfo><place><placeTerm type="code" authority="marccountry">xx</placeTerm></place><dateIssued encoding="marc">2014</dateIssued><issuance>monographic</issuance></originInfo><originInfo eventType="publication"><place><placeTerm type="text">[Leipzig]</placeTerm></place><publisher>top ten tv</publisher><dateIssued>[2014]</dateIssued></originInfo><language><languageTerm authority="iso639-2b" type="code">ger</languageTerm></language><physicalDescription><form authority="marccategory">electronic resource</form><form authority="marcsmd">remote</form><extent>1 Online-Ressource (1 Videodatei, 29:49) farbig</extent><form type="media" authority="rdamedia">Computermedien</form><form type="carrier" authority="rdacarrier">Online-Ressource</form></physicalDescription><targetAudience authority="marctarget">juvenile</targetAudience><note type="statement of responsibility" altRepGroup="00">ein Film von Dirk Schneider</note><note>Dokumentarfilm. Deutschland. 2014</note><relatedItem type="series"><titleInfo><title>MDR</title></titleInfo></relatedItem><relatedItem type="series"><titleInfo><title>Der Osten - entdecke wo du lebst</title></titleInfo></relatedItem><identifier type="oclc">946544758</identifier><recordInfo><descriptionStandard>rda</descriptionStandard><recordContentSource authority="marcorg">DE-576</recordContentSource><recordCreationDate encoding="marc">160304</recordCreationDate><recordChangeDate encoding="iso8601">20160510144338.0</recordChangeDate><recordIdentifier source="DE-576">457035137</recordIdentifier><recordOrigin>Converted from MARCXML to MODS version 3.6 using MARC21slim2MODS3-6.xsl
                (Revision 1.119 2018/06/21)</recordOrigin><languageOfCataloging><languageTerm authority="iso639-2b" type="code">ger</languageTerm></languageOfCataloging></recordInfo></mods>
PATCHED_MODS
my $unpatched_mods_obj = XML::LibXML->load_xml(string => $unpatched_mods);

my $unpatched_marcblob=<<'UNPATCHED_MARCBLOB';
<?xml version="1.0"?>
<record xmlns="http://www.loc.gov/MARC21/slim">
  <leader>     cgm a22        4500</leader>
  <controlfield tag="001">457035137</controlfield>
  <controlfield tag="003">DE-576</controlfield>
  <controlfield tag="005">20160510144338.0</controlfield>
  <controlfield tag="006">m     o  | |      </controlfield>
  <controlfield tag="007">cr uuu---uuuuu</controlfield>
  <controlfield tag="007">vu uuuuuu</controlfield>
  <controlfield tag="008">160304s2014    xx ger c</controlfield>
  <datafield tag="035" ind1=" " ind2=" ">
    <subfield code="a">(DE-627)1655506501</subfield>
  </datafield>
  <datafield tag="035" ind1=" " ind2=" ">
    <subfield code="a">(DE-576)457035137</subfield>
  </datafield>
  <datafield tag="035" ind1=" " ind2=" ">
    <subfield code="a">(DE-599)BSZ457035137</subfield>
  </datafield>
  <datafield tag="035" ind1=" " ind2=" ">
    <subfield code="a">(OCoLC)946544758</subfield>
  </datafield>
  <datafield tag="040" ind1=" " ind2=" ">
    <subfield code="a">DE-576</subfield>
    <subfield code="b">ger</subfield>
    <subfield code="c">DE-576</subfield>
    <subfield code="e">rda</subfield>
  </datafield>
  <datafield tag="041" ind1=" " ind2=" ">
    <subfield code="a">ger</subfield>
  </datafield>
  <datafield tag="245" ind1="1" ind2="4">
    <subfield code="a">Der Fichtelberg</subfield>
    <subfield code="b">Berg der unbekannten Rekorrde</subfield>
    <subfield code="c">ein Film von Dirk Schneider</subfield>
  </datafield>
  <datafield tag="264" ind1=" " ind2="1">
    <subfield code="a">[Leipzig]</subfield>
    <subfield code="b">top ten tv</subfield>
    <subfield code="c">[2014]</subfield>
  </datafield>
  <datafield tag="264" ind1=" " ind2="4">
    <subfield code="c">&#xA9; 2014</subfield>
  </datafield>
  <datafield tag="300" ind1=" " ind2=" ">
    <subfield code="a">1 Online-Ressource (1 Videodatei, 29:49)</subfield>
    <subfield code="b">farbig</subfield>
  </datafield>
  <datafield tag="336" ind1=" " ind2=" ">
    <subfield code="a">zweidimensionales bewegtes Bild</subfield>
    <subfield code="b">tdi</subfield>
    <subfield code="2">rdacontent</subfield>
  </datafield>
  <datafield tag="337" ind1=" " ind2=" ">
    <subfield code="a">Computermedien</subfield>
    <subfield code="b">c</subfield>
    <subfield code="2">rdamedia</subfield>
  </datafield>
  <datafield tag="338" ind1=" " ind2=" ">
    <subfield code="a">Online-Ressource</subfield>
    <subfield code="b">cr</subfield>
    <subfield code="2">rdacarrier</subfield>
  </datafield>
  <datafield tag="490" ind1="0" ind2=" ">
    <subfield code="a">MDR</subfield>
  </datafield>
  <datafield tag="490" ind1="0" ind2=" ">
    <subfield code="a">Der Osten - entdecke wo du lebst</subfield>
  </datafield>
  <datafield tag="500" ind1=" " ind2=" ">
    <subfield code="a">Dokumentarfilm. Deutschland. 2014</subfield>
  </datafield>
  <datafield tag="591" ind1=" " ind2=" ">
    <subfield code="a">Fernsehmitschnitt (SWB)</subfield>
  </datafield>
  <datafield tag="655" ind1=" " ind2="7">
    <subfield code="a">Film</subfield>
    <subfield code="0">(DE-588)4017102-4</subfield>
    <subfield code="0">(DE-627)104559683</subfield>
    <subfield code="0">(DE-576)208918531</subfield>
    <subfield code="2">gnd-content</subfield>
  </datafield>
  <datafield tag="700" ind1="1" ind2=" ">
    <subfield code="a">Schneider, Dirk</subfield>
    <subfield code="e">FilmemacherIn</subfield>
    <subfield code="0">(DE-627)1235502279</subfield>
    <subfield code="0">(DE-576)165502274</subfield>
    <subfield code="4">fmk</subfield>
  </datafield>
  <datafield tag="935" ind1=" " ind2=" ">
    <subfield code="c">vide</subfield>
  </datafield>
  <datafield tag="937" ind1=" " ind2=" ">
    <subfield code="a">Dokumentarfilm</subfield>
    <subfield code="b">Deutschland</subfield>
    <subfield code="c">2014</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">000  xxxxxcx  a22     zn 4500</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">001 901795887</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">003  DE-576</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">004  457035137</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">005  20160510125331</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">008  160304||||||||||||||||ger|||||||</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">040  </subfield>
    <subfield code="a">DE-14</subfield>
    <subfield code="c">DE-576</subfield>
    <subfield code="d">DE-14</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">852  </subfield>
    <subfield code="z">Fernsehmitschnitt: MDR, 04.02.2014. - Beilage</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">852  </subfield>
    <subfield code="a">DE-14</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">852 1</subfield>
    <subfield code="9">00</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">866  </subfield>
    <subfield code="x">ddsu,pn</subfield>
  </datafield>
</record>
UNPATCHED_MARCBLOB

my $patched_marcblob =<<'PATCHED_MARCBLOB';
<?xml version="1.0"?>
<record xmlns="http://www.loc.gov/MARC21/slim">
  <leader>     cgm a22        4500</leader>
  <controlfield tag="001">457035137</controlfield>
  <controlfield tag="003">DE-576</controlfield>
  <controlfield tag="005">20160510144338.0</controlfield>
  <controlfield tag="006">m     o  | |      </controlfield>
  <controlfield tag="007">cr uuu---uuuuu</controlfield>
  <controlfield tag="007">vu uuuuuu</controlfield>
  <controlfield tag="008">160304s2014    xx ger c</controlfield>
  <datafield tag="035" ind1=" " ind2=" ">
    <subfield code="a">(DE-627)1655506501</subfield>
  </datafield>
  <datafield tag="035" ind1=" " ind2=" ">
    <subfield code="a">(DE-576)457035137</subfield>
  </datafield>
  <datafield tag="035" ind1=" " ind2=" ">
    <subfield code="a">(DE-599)BSZ457035137</subfield>
  </datafield>
  <datafield tag="035" ind1=" " ind2=" ">
    <subfield code="a">(OCoLC)946544758</subfield>
  </datafield>
  <datafield tag="040" ind1=" " ind2=" ">
    <subfield code="a">DE-576</subfield>
    <subfield code="b">ger</subfield>
    <subfield code="c">DE-576</subfield>
    <subfield code="e">rda</subfield>
  </datafield>
  <datafield tag="041" ind1=" " ind2=" ">
    <subfield code="a">ger</subfield>
  </datafield>
  <datafield tag="245" ind1="1" ind2="4">
    <subfield code="a">Der Fichtelberg</subfield>
    <subfield code="b">Berg der unbekannten Rekorrde</subfield>
    <subfield code="c">ein Film von Dirk Schneider</subfield>
  </datafield>
  <datafield tag="264" ind1=" " ind2="1">
    <subfield code="a">[Leipzig]</subfield>
    <subfield code="b">top ten tv</subfield>
    <subfield code="c">[2014]</subfield>
  </datafield>
  <datafield tag="264" ind1=" " ind2="4">
    <subfield code="c">&#xA9; 2014</subfield>
  </datafield>
  <datafield tag="300" ind1=" " ind2=" ">
    <subfield code="a">1 Online-Ressource (1 Videodatei, 29:49)</subfield>
    <subfield code="b">farbig</subfield>
  </datafield>
  <datafield tag="336" ind1=" " ind2=" ">
    <subfield code="a">zweidimensionales bewegtes Bild</subfield>
    <subfield code="b">tdi</subfield>
    <subfield code="2">rdacontent</subfield>
  </datafield>
  <datafield tag="337" ind1=" " ind2=" ">
    <subfield code="a">Computermedien</subfield>
    <subfield code="b">c</subfield>
    <subfield code="2">rdamedia</subfield>
  </datafield>
  <datafield tag="338" ind1=" " ind2=" ">
    <subfield code="a">Online-Ressource</subfield>
    <subfield code="b">cr</subfield>
    <subfield code="2">rdacarrier</subfield>
  </datafield>
  <datafield tag="490" ind1="0" ind2=" ">
    <subfield code="a">MDR</subfield>
  </datafield>
  <datafield tag="490" ind1="0" ind2=" ">
    <subfield code="a">Der Osten - entdecke wo du lebst</subfield>
  </datafield>
  <datafield tag="500" ind1=" " ind2=" ">
    <subfield code="a">Dokumentarfilm. Deutschland. 2014</subfield>
  </datafield>
  <datafield tag="591" ind1=" " ind2=" ">
    <subfield code="a">Fernsehmitschnitt (SWB)</subfield>
  </datafield>
  <datafield tag="655" ind1=" " ind2="7">
    <subfield code="a">Film</subfield>
    <subfield code="0">(DE-588)4017102-4</subfield>
    <subfield code="0">(DE-627)104559683</subfield>
    <subfield code="0">(DE-576)208918531</subfield>
    <subfield code="2">gnd-content</subfield>
  </datafield>
  <datafield tag="700" ind1="1" ind2=" ">
    <subfield code="a">Schneider, Dirk</subfield>
    <subfield code="e">FilmemacherIn</subfield>
    <subfield code="0">(DE-627)1235502279</subfield>
    <subfield code="0">(DE-576)165502274</subfield>
    <subfield code="4">fmk</subfield>
  </datafield>
  <datafield tag="935" ind1=" " ind2=" ">
    <subfield code="c">vide</subfield>
  </datafield>
  <datafield tag="937" ind1=" " ind2=" ">
    <subfield code="a">Dokumentarfilm</subfield>
    <subfield code="b">Deutschland</subfield>
    <subfield code="c">2014</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">000  xxxxxcx  a22     zn 4500</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">001 901795887</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">003  DE-576</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">004  457035137</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">005  20160510125331</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">008  160304||||||||||||||||ger|||||||</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">040  </subfield>
    <subfield code="a">DE-14</subfield>
    <subfield code="c">DE-576</subfield>
    <subfield code="d">DE-14</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">852  </subfield>
    <subfield code="z">Fernsehmitschnitt: MDR, 04.02.2014. - Beilage</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">852  </subfield>
    <subfield code="a">DE-14</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">852 1</subfield>
    <subfield code="9">00</subfield>
  </datafield>
  <datafield tag="LOK" ind1=" " ind2=" ">
    <subfield code="0">866  </subfield>
    <subfield code="x">ddsu,pn</subfield>
  </datafield>
</record>
PATCHED_MARCBLOB
my $unpatched_marcblob_obj = XML::LibXML->load_xml(string => $unpatched_marcblob);

my $unpatched_marc21slimutils = <<'UNPATCHED_MARC21SLIMUTIL';
<?xml version='1.0'?>
<xsl:stylesheet version="1.0" xmlns:marc="http://www.loc.gov/MARC21/slim"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

    <!-- 08/08/08: tmee added corrected chopPunctuation templates for 260c -->
    <!-- 08/19/04: ntra added "marc:" prefix to datafield element -->
    <!-- 12/14/07: ntra added url encoding template -->
    <!-- url encoding -->

    <xsl:variable name="ascii">
        <xsl:text> !"#$%&amp;'()*+,-./0123456789:;&lt;=&gt;?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~</xsl:text>
    </xsl:variable>

    <xsl:variable name="latin1">
        <xsl:text> ¡¢£¤¥¦§¨©ª«¬­®¯°±²³´µ¶·¸¹º»¼½¾¿ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖ×ØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõö÷øùúûüýþÿ</xsl:text>
    </xsl:variable>
    <!-- Characters that usually don't need to be escaped -->
    <xsl:variable name="safe">
        <xsl:text>!'()*-.0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz~</xsl:text>
    </xsl:variable>

    <xsl:variable name="hex">0123456789ABCDEF</xsl:variable>


    <xsl:template name="datafield">
        <xsl:param name="tag"/>
        <xsl:param name="ind1">
            <xsl:text> </xsl:text>
        </xsl:param>
        <xsl:param name="ind2">
            <xsl:text> </xsl:text>
        </xsl:param>
        <xsl:param name="subfields"/>
        <xsl:element name="marc:datafield">
            <xsl:attribute name="tag">
                <xsl:value-of select="$tag"/>
            </xsl:attribute>
            <xsl:attribute name="ind1">
                <xsl:value-of select="$ind1"/>
            </xsl:attribute>
            <xsl:attribute name="ind2">
                <xsl:value-of select="$ind2"/>
            </xsl:attribute>
            <xsl:copy-of select="$subfields"/>
        </xsl:element>
    </xsl:template>

    <xsl:template name="subfieldSelect">
        <xsl:param name="codes">abcdefghijklmnopqrstuvwxyz</xsl:param>
        <xsl:param name="delimeter">
            <xsl:text> </xsl:text>
        </xsl:param>
        <xsl:variable name="str">
            <xsl:for-each select="marc:subfield">
                <xsl:if test="contains($codes, @code)">
                    <xsl:value-of select="text()"/>
                    <xsl:value-of select="$delimeter"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>
        <xsl:value-of select="substring($str,1,string-length($str)-string-length($delimeter))"/>
    </xsl:template>

    <xsl:template name="buildSpaces">
        <xsl:param name="spaces"/>
        <xsl:param name="char">
            <xsl:text> </xsl:text>
        </xsl:param>
        <xsl:if test="$spaces>0">
            <xsl:value-of select="$char"/>
            <xsl:call-template name="buildSpaces">
                <xsl:with-param name="spaces" select="$spaces - 1"/>
                <xsl:with-param name="char" select="$char"/>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>

    <xsl:template name="chopPunctuation">
        <xsl:param name="chopString"/>
        <xsl:param name="punctuation">
            <xsl:text>.:,;/ </xsl:text>
        </xsl:param>
        <xsl:variable name="length" select="string-length($chopString)"/>
        <xsl:choose>
            <xsl:when test="$length=0"/>
            <xsl:when test="contains($punctuation, substring($chopString,$length,1))">
                <xsl:call-template name="chopPunctuation">
                    <xsl:with-param name="chopString" select="substring($chopString,1,$length - 1)"/>
                    <xsl:with-param name="punctuation" select="$punctuation"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:when test="not($chopString)"/>
            <xsl:otherwise>
                <xsl:value-of select="$chopString"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="chopPunctuationFront">
        <xsl:param name="chopString"/>
        <xsl:variable name="length" select="string-length($chopString)"/>
        <xsl:choose>
            <xsl:when test="$length=0"/>
            <xsl:when test="contains('.:,;/[ ', substring($chopString,1,1))">
                <xsl:call-template name="chopPunctuationFront">
                    <xsl:with-param name="chopString" select="substring($chopString,2,$length - 1)"
                    />
                </xsl:call-template>
            </xsl:when>
            <xsl:when test="not($chopString)"/>
            <xsl:otherwise>
                <xsl:value-of select="$chopString"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="chopPunctuationBack">
        <xsl:param name="chopString"/>
        <xsl:param name="punctuation">
            <xsl:text>.:,;/] </xsl:text>
        </xsl:param>
        <xsl:variable name="length" select="string-length($chopString)"/>
        <xsl:choose>
            <xsl:when test="$length=0"/>
            <xsl:when test="contains($punctuation, substring($chopString,$length,1))">
                <xsl:call-template name="chopPunctuation">
                    <xsl:with-param name="chopString" select="substring($chopString,1,$length - 1)"/>
                    <xsl:with-param name="punctuation" select="$punctuation"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:when test="not($chopString)"/>
            <xsl:otherwise>
                <xsl:value-of select="$chopString"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- nate added 12/14/2007 for lccn.loc.gov: url encode ampersand, etc. -->
    <xsl:template name="url-encode">

        <xsl:param name="str"/>

        <xsl:if test="$str">
            <xsl:variable name="first-char" select="substring($str,1,1)"/>
            <xsl:choose>
                <xsl:when test="contains($safe,$first-char)">
                    <xsl:value-of select="$first-char"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:variable name="codepoint">
                        <xsl:choose>
                            <xsl:when test="contains($ascii,$first-char)">
                                <xsl:value-of
                                    select="string-length(substring-before($ascii,$first-char)) + 32"
                                />
                            </xsl:when>
                            <xsl:when test="contains($latin1,$first-char)">
                                <xsl:value-of
                                    select="string-length(substring-before($latin1,$first-char)) + 160"/>
                                <!-- was 160 -->
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:message terminate="no">Warning: string contains a character
                                    that is out of range! Substituting "?".</xsl:message>
                                <xsl:text>63</xsl:text>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="hex-digit1"
                        select="substring($hex,floor($codepoint div 16) + 1,1)"/>
                    <xsl:variable name="hex-digit2" select="substring($hex,$codepoint mod 16 + 1,1)"/>
                    <!-- <xsl:value-of select="concat('%',$hex-digit2)"/> -->
                    <xsl:value-of select="concat('%',$hex-digit1,$hex-digit2)"/>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:if test="string-length($str) &gt; 1">
                <xsl:call-template name="url-encode">
                    <xsl:with-param name="str" select="substring($str,2)"/>
                </xsl:call-template>
            </xsl:if>
        </xsl:if>
    </xsl:template>
</xsl:stylesheet>
<!-- Stylus Studio meta-information - (c)1998-2002 eXcelon Corp.
<metaInformation>
<scenarios/><MapperInfo srcSchemaPath="" srcSchemaRoot="" srcSchemaPathIsRelative="yes" srcSchemaInterpretAsXML="no" destSchemaPath="" destSchemaRoot="" destSchemaPathIsRelative="yes" destSchemaInterpretAsXML="no"/>
</metaInformation>
-->
UNPATCHED_MARC21SLIMUTIL

my $noppn_mods_res = "<mods version=\"3.6\"
    xmlns=\"http://www.loc.gov/mods/v3\"
    xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"
    xsi:schemaLocation=\"http://www.loc.gov/mods/v3 http://www.loc.gov/standards/mods/v3/mods-3-6.xsd\">
    <identifier>string</identifier>
</mods>
";

# preparation
my $xsl_path = path(__FILE__)->parent->parent->child('xsl');
# input
my $export_dir_kitodo_path = path(__FILE__)->parent->parent->child('export_dir_kitodo');
if(! -d $export_dir_kitodo_path) { mkdir $export_dir_kitodo_path or die "Error creating directory: $export_dir_kitodo_path"; }
my $input_path = path(__FILE__)->parent->parent->child('export_dir_kitodo')->child('bagit');
if(! -d $input_path) { mkdir $input_path or die "Error creating directory: $input_path"; }
# output
my $output_path = path(__FILE__)->parent->parent->child('tmp')->child('bagit');
if(! -d $output_path->parent) { mkdir $output_path->parent or die "Error creating directory: tmp";}
if(! -d $output_path) { mkdir $output_path or die "Error creating directory: $output_path";}
# create ie
my $ie = 'test';
my @addBagInfo;
push @addBagInfo, {'SLUBArchiv-sipVersion' => 'v2020.1'};
push @addBagInfo, {'Author' => 'Taras Schevchenko'};
my $ie_input_path = $input_path . '/' . $ie;
my $ie_output_path = $output_path . '/' . $ie;
my $file_name = $ie_output_path . '/data/' . 'test.txt';
if(! -d $input_path->child("test")){ mkdir $input_path->child("test") or die "Error creating directory: " . $input_path->child("test"); }
if(! -d $input_path->child("test")->child("data")){ mkdir $input_path->child("test")->child("data") or die "Error creating directory: " . $input_path->child("test")->("data"); }
if(! -f $input_path->child("test")->child("data")->child("test.txt")) { 
    open(my $fh, '>:encoding(UTF-8)', $input_path->child("test")->child("data")->child("test.txt")) or die "Could not open file " . $input_path->child("test")->child("data")->child("test.txt") . " $!";
    print $fh "test file for payload.\n";
    close $fh; 
}
if(! -d $input_path->child("test")->child("meta")){ mkdir $input_path->child("test")->child("meta") or die "Error creating directory: " . $input_path->child("test")->child("meta"); }
# copy Fallbeispiele-*.xml (as rights.xml) file to output_path/meta
my $rightsXMLPath = path(__FILE__)->parent->realpath->parent->child("export_dir_kitodo")->child("bagit")->child("rights");    # absolute path
File::Copy::Recursive::dircopy($rightsXMLPath, "$ie_input_path/meta") or die "Could not perform dircopy() of $rightsXMLPath to $ie_input_path/meta: $!";
# copy other meta files to output_path/meta
my $otherMetaPath = path(__FILE__)->parent->realpath->parent->child("export_dir_kitodo")->child("bagit")->child("meta");    # absolute path
File::Copy::Recursive::dircopy($otherMetaPath, "$ie_input_path/meta") or die "Could not perform dircopy() of $rightsXMLPath to $ie_input_path/meta: $!";
# delete bagit from last session
rmtree $ie_output_path;
# copy new ie to output_path
File::Copy::Recursive::dircopy($ie_input_path, $ie_output_path) or die "Could not perform dircopy() of $ie_input_path to $ie_output_path: $!";

#~ structure of input output folders for tests
#~ .
#~ ├── export_dir_kitodo
#~ │   └─── bagit
#~ │      └── test    
#~ │    
#~ └── tmp
#~       └── bagit
#~         └── test

my $ua = LWP::UserAgent->new;
my $request = HTTP::Request->new('GET' => "https://sru.bsz-bw.de/swbf");
my $response = $ua->request($request);
my $marc21slimutils_file = $xsl_path->child('MARC21slimUtils.xsl');
my $useragent_obj = LWP::UserAgent->new;
$useragent_obj->agent("MyApp/0.1 ");
$useragent_obj->timeout(3600); #1h

# ensure no dir exists, then run test
if($xsl_path->is_dir){ $xsl_path->remove_tree; }

### tests
BEGIN { use_ok("SLUB::LZA::SIPBuilderBagIt"); }
#
is(SLUB::LZA::SIPBuilderBagIt::check_xsl_directory(), $xsl_path->absolute, "check_xsl_directory(), return value if not exist");
ok($xsl_path->is_dir, "check_xsl_directory(), created if not exist");
is(SLUB::LZA::SIPBuilderBagIt::check_xsl_directory(), $xsl_path->absolute, "check_xsl_directory(), return value if exist");
ok($xsl_path->is_dir, "check_xsl_directory(), untouched if exist");
#
SKIP: {
    skip "No response from server https://sru.bsz-bw.de/swb", 1 unless ! $response->is_error;
    like(SLUB::LZA::SIPBuilderBagIt::get_mods_from("https://sru.bsz-bw.de/swb", "457035137", "pica.swn", "marcxmlvbos"), qr//, "get_mods_from()");
};
throws_ok{ SLUB::LZA::SIPBuilderBagIt::get_mods_from("", "457035137", "pica.swn", "marcxmlvbos"); } qr/invalid parameters/, "get_mods_from(), invalid parameters";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::get_mods_from("https://sru.bsz-bw.de/swb", "", "pica.swn", "marcxmlvbos"); } qr/invalid parameters/, "get_mods_from(), invalid parameters";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::get_mods_from("https://sru.bsz-bw.de/swb", "457035137", "", "marcxmlvbos"); } qr/invalid parameters/, "get_mods_from(), invalid parameters";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::get_mods_from("https://sru.bsz-bw.de/swb", "457035137", "pica.swn", ""); } qr/invalid parameters/, "get_mods_from(), invalid parameters";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::get_mods_from(undef, "457035137", "pica.swn", "marcxmlvbos"); } qr/url not defined/, "get_mods_from(\$url, \$ppn, \$key, \$schema), url undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::get_mods_from("https://sru.bsz-bw.de/swb", undef, "pica.swn", "marcxmlvbos"); } qr/ppn not defined/, "get_mods_from(\$url, \$ppn, \$key, \$schema), ppn undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::get_mods_from("https://sru.bsz-bw.de/swb", "457035137", undef, "marcxmlvbos"); } qr/key not defined/, "get_mods_from(\$url, \$ppn, \$key, \$schema), key undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::get_mods_from("https://sru.bsz-bw.de/swb", "457035137", "pica.swn", undef); } qr/schema not defined/, "get_mods_from(\$url, \$ppn, \$key, \$schema), schema undef";
#
ok(SLUB::LZA::SIPBuilderBagIt::write_file($marc21slimutils_file, $unpatched_marc21slimutils), "write_file()");
throws_ok{ SLUB::LZA::SIPBuilderBagIt::write_file("", $unpatched_marc21slimutils); } qr/invalid parameters/, "write_file(), invalid parameters";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::write_file($marc21slimutils_file, ""); } qr/invalid parameters/, "write_file(), invalid parameters";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::write_file(undef, $unpatched_marc21slimutils); } qr/filename not defined/, "write_file(\$filename, \$value), filename undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::write_file($marc21slimutils_file, undef); } qr/value not defined/, "write_file(\$filename, \$value), value undef";
#
is(SLUB::LZA::SIPBuilderBagIt::patch_mods($unpatched_mods_obj), $patched_mods, "patch_mods()");
throws_ok{ SLUB::LZA::SIPBuilderBagIt::patch_mods(""); } qr/invalid parameters/, "patch_mods(), invalid parameters";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::patch_mods(undef); } qr/modsobject not defined/, "patch_mods(\$modsobj), modsobj undef";
#
is(SLUB::LZA::SIPBuilderBagIt::patch_marc_response($unpatched_marcblob_obj), $patched_marcblob, "patch_marc_response()");
throws_ok{ SLUB::LZA::SIPBuilderBagIt::patch_marc_response(""); } qr/invalid parameters/, "patch_marc_response(), invalid parameters";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::patch_marc_response(undef); } qr/marcobject not defined/, "patch_marc_response(\$marcobj), marcobj undef";
#
is(SLUB::LZA::SIPBuilderBagIt::check_marc21_utility($xsl_path, $useragent_obj), $marc21slimutils_file , "check_marc21_utility()");
throws_ok{ SLUB::LZA::SIPBuilderBagIt::check_marc21_utility("", $useragent_obj); } qr/invalid parameters/, "check_marc21_utility(), invalid parameters";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::check_marc21_utility($xsl_path, ""); } qr/invalid parameters/, "check_marc21_utility(), invalid parameters";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::check_marc21_utility(undef, $useragent_obj); } qr/xsl directory not defined/, "check_marc21_utility(\$xsl_dir, \$useragent), xsl_dir undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::check_marc21_utility($xsl_path, undef); } qr/user agent not defined/, "check_marc21_utility(\$xsl_dir, \$useragent), useragent undef";
#
is(SLUB::LZA::SIPBuilderBagIt::check_marc21_mods_xsl($xsl_path, $useragent_obj), 'xsl/MARC21slim2MODS3-6.patched.xsl' , "check_marc21_mods_xsl()");
throws_ok{ SLUB::LZA::SIPBuilderBagIt::check_marc21_mods_xsl("", $useragent_obj); } qr/invalid parameters/, "check_marc21_mods_xsl(), invalid parameters";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::check_marc21_mods_xsl($xsl_path, ""); } qr/invalid parameters/, "check_marc21_mods_xsl(), invalid parameters";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::check_marc21_mods_xsl(undef, $useragent_obj); } qr/xsl directory not defined/, "check_marc21_mods_xsl(\$xsl_dir, \$useragent), xsl_dir undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::check_marc21_mods_xsl($xsl_path, undef); } qr/user agent not defined/, "check_marc21_mods_xsl(\$xsl_dir, \$useragent), useragent undef";
#
SKIP: {
    skip "No response from server https://sru.bsz-bw.de/swb", 1 unless ! $response->is_error;
    like(SLUB::LZA::SIPBuilderBagIt::prepare_mods_section_with_ppn("457035137"), qr//, "prepare_mods_section_with_ppn()");
};
throws_ok{ SLUB::LZA::SIPBuilderBagIt::prepare_mods_section_with_ppn(""); } qr/invalid parameters/, "prepare_mods_section_with_ppn(), invalid parameters";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::prepare_mods_section_with_ppn(undef); } qr/ppn not defined/, "prepare_mods_section_with_ppn(\$ppn), ppn undef";
#
is(SLUB::LZA::SIPBuilderBagIt::prepare_mods_section_with_noppn("string"), $noppn_mods_res, "prepare_mods_section_with_noppn()");
throws_ok{ SLUB::LZA::SIPBuilderBagIt::prepare_mods_section_with_noppn(""); } qr/noppn is empty string!/, "prepare_mods_section_with_noppn(), noppn is empty string!";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::prepare_mods_section_with_noppn(undef); } qr/noppn is not defined/, "prepare_mods_section_with_noppn(\$noppn), noppn undef";
# test new
ok(SLUB::LZA::SIPBuilderBagIt::create_slub_bagit($ie_output_path, \@addBagInfo), "create_slub_bagit()");
throws_ok{ SLUB::LZA::SIPBuilderBagIt::create_slub_bagit(undef, \@addBagInfo); } qr/ie_path is not defined/, "create_slub_bagit(\$ie_path, \$refAddBagInfo), ie_path undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::create_slub_bagit($ie_output_path, undef); } qr/array of hashes not defined for bag-info.txt/, "create_slub_bagit(\$ie_path, \$refAddBagInfo), refAddBagInfo undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::create_slub_bagit("", \@addBagInfo); } qr/path is not directory/, "create_slub_bagit(), invalid parameters";
#
is(SLUB::LZA::SIPBuilderBagIt::getBagItName($ie_output_path), $ie, "getBagItName()");
throws_ok{ SLUB::LZA::SIPBuilderBagIt::getBagItName(undef); } qr/path for getting bagit name is not defined/, "getBagItName(\$bagit_name), bagit_name undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::getBagItName(""); } qr/path is not directory/, "getBagItName(), invalid parameters";
#
is(SLUB::LZA::SIPBuilderBagIt::getFileName($file_name), "test.txt", "getFileName()");
throws_ok{ SLUB::LZA::SIPBuilderBagIt::getFileName(undef); } qr/path with file name is not defined/, "getFileName(\$file_name), file_name undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::getFileName(""); } qr/invalid parameters/, "getFileName(), invalid parameters";
#
is(SLUB::LZA::SIPBuilderBagIt::save_option("copy", $ie_input_path, $ie_output_path, "ID-10000000_2020-01-01_20-00-00"), $ie_output_path . "/ID-10000000_2020-01-01_20-00-00", "save_option()");
rmtree $ie_output_path . "/ID-10000000_2020-01-01_20-00-00";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::save_option(undef, $ie_input_path, $ie_output_path, "ID-10000000_2020-01-01_20-00-00"); } qr/save option not defined/, "save_option(\$save, \$directory, \$output, \$bagNewDir), save undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::save_option("copy", undef, $ie_output_path, "ID-10000000_2020-01-01_20-00-00"); } qr/IE directory path not defined/, "save_option(\$save, \$directory, \$output, \$bagNewDir), directory undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::save_option("copy", $ie_input_path, undef, "ID-10000000_2020-01-01_20-00-00"); } qr/output path not defined/, "save_option(\$save, \$directory, \$output, \$bagNewDir), for \$save=copy directory undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::save_option("move", $ie_input_path, undef, "ID-10000000_2020-01-01_20-00-00"); } qr/output path not defined/, "save_option(\$save, \$directory, \$output, \$bagNewDir), for \$save=move directory undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::save_option("copy", $ie_input_path, $ie_output_path, undef); } qr/sip root dir is not defined/, "save_option(\$save, \$directory, \$output, \$bagNewDir), bagNewDir undef";
#
is(SLUB::LZA::SIPBuilderBagIt::getPayloadFiles($ie_input_path), 2, "getPayloadFiles()"); # meta, data
throws_ok{ SLUB::LZA::SIPBuilderBagIt::getPayloadFiles(undef); } qr/path is not defined/, "getPayloadFiles(\$import_dir), import_dir undef";
#
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-01.xml"), 1, "validateRightsXML() Fallbeispiel-01.xml");
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-02.xml"), 1, "validateRightsXML() Fallbeispiel-02.xml");
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-03.xml"), 1, "validateRightsXML() Fallbeispiel-03.xml");
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-04.xml"), 1, "validateRightsXML() Fallbeispiel-04.xml");
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-05.xml"), 1, "validateRightsXML() Fallbeispiel-05.xml");
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-06.xml"), 1, "validateRightsXML() Fallbeispiel-06.xml");
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-07.xml"), 1, "validateRightsXML() Fallbeispiel-07.xml");
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-08a-undef.xml"), 1, "validateRightsXML() Fallbeispiel-08a-undef.xml");
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-09.xml"), 1, "validateRightsXML() Fallbeispiel-09.xml");
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-10.xml"), 1, "validateRightsXML() Fallbeispiel-10.xml");
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-11.xml"), 1, "validateRightsXML() Fallbeispiel-11.xml");
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-12.xml"), 1, "validateRightsXML() Fallbeispiel-12.xml");
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-13.xml"), 1, "validateRightsXML() Fallbeispiel-13.xml");
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-14.xml"), 1, "validateRightsXML() Fallbeispiel-14.xml");
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-15.xml"), 1, "validateRightsXML() Fallbeispiel-15.xml");
is(SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "Fallbeispiel-16.xml"), 1, "validateRightsXML() Fallbeispiel-16.xml");
throws_ok{ SLUB::LZA::SIPBuilderBagIt::validateRightsXML(undef, "rights.xml"); } qr/ie_path to rights.xsd not defined/, "validateRightsXML(\$ie_path, \$rightsName), ie_path undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, undef); } qr/file name not defined/, "validateRightsXML(\$ie_path, \$rightsName), rightsName undef";
throws_ok{ SLUB::LZA::SIPBuilderBagIt::validateRightsXML($ie_input_path, "mods.xml"); } qr/failed validation/, "validateRightsXML(\$ie_path, \$rightsName), not valid rights.xml";


1;
